@@ -6,7 +6,7 @@ copyright 2007,2012 by Flavio Codeco Coelho
 Licensed under the GPL.
 """
 import locale, os, pylab
-from osgeo import ogr
+from osgeo import ogr, osr, gdal
 from xml.dom import minidom, Node
 from matplotlib.colors import rgb2hex, LogNorm
 from matplotlib.colors import Normalize
@@ -25,17 +25,32 @@ class World:
         self.geocfield = geocfield
         self.namefield = namefield
         self.outdir = outdir
-        self.ds = ogr.Open(filename)
+        try:
+            self.ds = ogr.Open(filename)
+        except AttributeError:
+            raise AttributeError("Could not open the Shapefile {}. Please check if the path is correct.".format(filename))
         self.name = self.ds.GetName()
         self.driver = self.ds.GetDriver()
-        self.centroids = []#centroid list (x,y,z) tuples
-        self.centdict = {} #keys are geocode, values are (x,y,z) tuples
-        self.geomdict = {} #keys are geocode, values are geometries
-        self.namedict = {} #keys are geocode, values are locality names
-        self.nlist = [] #nodelist: feature objects
-        self.nodesource = False #True if Node datasource has been created
-        self.edgesource = False #True if Edge datasource has been created
-        self.datasource = False #True if Data datasource has been created
+        layer = self.ds.GetLayer()
+        if layer.GetSpatialRef() is None:
+            self.in_spatial_ref = osr.SpatialReference()
+            self.in_spatial_ref.ImportFromEPSG(4326)
+        else:
+            epsg = layer.GetEPSG()
+            self.in_spatial_ref = osr.SpatialReference()
+            self.in_spatial_ref.ImportFromEPSG(epsg)
+
+        self.out_spatial_ref = osr.SpatialReference()
+        self.out_spatial_ref.ImportFromEPSG(4326)
+        self.coord_trans = osr.CoordinateTransformation(self.in_spatial_ref, self.out_spatial_ref)
+        self.centroids = []  # centroid list (x,y,z) tuples
+        self.centdict = {}  # keys are geocode, values are (x,y,z) tuples
+        self.geomdict = {}  # keys are geocode, values are geometries
+        self.namedict = {}  # keys are geocode, values are locality names
+        self.nlist = []  # nodelist: feature objects
+        self.nodesource = False  # True if Node datasource has been created
+        self.edgesource = False  # True if Edge datasource has been created
+        self.datasource = False  # True if Data datasource has been created
         self.layerlist = self.get_layer_list()
 
     def get_layer_list(self):
@@ -87,17 +102,17 @@ class World:
         f = l.GetNextFeature()
         while f is not None:
             g = f.GetGeometryRef()
+            g.Transform(self.coord_trans)  # Transforming to EPSG4326 coordinate system
             self.geomdict[f.GetFieldAsInteger(self.geocfield)] = g
             try:
                 c = g.Centroid()
                 self.nlist.append(f)
                 self.centdict[f.GetFieldAsInteger(self.geocfield)] = (c.GetX(), c.GetY(), c.GetZ())
                 self.namedict[f.GetFieldAsInteger(self.geocfield)] = f.GetField(self.namefield)
-            except: #in case feature is not a polygon
+            except:  # in case feature is not a polygon
                 print  f.GetFID(), g.GetGeometryType()
             f = l.GetNextFeature()
-            #print (2600501 in self.centdict)
-
+            # print (2600501 in self.centdict)
 
     def create_node_layer(self):
         """
@@ -117,7 +132,7 @@ class World:
             dsn = self.driver.CreateDataSource(os.path.join(self.outdir, 'Nodes.shp'))
             self.nodesource = dsn
             nl = dsn.CreateLayer("nodes", geom_type=ogr.wkbPoint)
-            #Create the fields
+            # Create the fields
         fi1 = ogr.FieldDefn("name")
         fi2 = ogr.FieldDefn("geocode", field_type=ogr.OFTInteger)
         fi3 = ogr.FieldDefn("x", field_type=ogr.OFTString)
@@ -126,7 +141,7 @@ class World:
         nl.CreateField(fi2)
         nl.CreateField(fi3)
         nl.CreateField(fi4)
-        #Add the features (points)
+        # Add the features (points)
         for f in self.nlist:
             gc = f.GetFieldAsInteger(self.geocfield)
             x = self.centdict[gc][0]
@@ -142,6 +157,9 @@ class World:
             fe.SetGeometryDirectly(pt)
             nl.CreateFeature(fe)
         nl.SyncToDisk()
+        self.out_spatial_ref.MorphToESRI()
+        with open('Nodes.prj', 'w') as fobj:
+            fobj.write(self.out_spatial_ref.ExportToWkt())
 
     def create_edge_layer(self, elist):
         """
@@ -158,7 +176,7 @@ class World:
             dse = self.driver.CreateDataSource(os.path.join(self.outdir, 'Edges.shp'))
             self.edgesource = dse
             el = dse.CreateLayer("edges", geom_type=ogr.wkbLineString)
-            #Create the fields
+            # Create the fields
         fi1 = ogr.FieldDefn("s_geocode", field_type=ogr.OFTInteger)
         fi2 = ogr.FieldDefn("d_geocode", field_type=ogr.OFTInteger)
         fi3 = ogr.FieldDefn("flowSD", field_type=ogr.OFTReal)
@@ -169,9 +187,9 @@ class World:
         el.CreateField(fi2)
         el.CreateField(fi3)
         el.CreateField(fi4)
-        #Add the features (lines)
+        # Add the features (lines)
         for e in elist:
-            #print "setting edge fields"
+            # print "setting edge fields"
             fe = ogr.Feature(el.GetLayerDefn())
             fe.SetField('s_geocode', e[0])
             fe.SetField('d_geocode', e[1])
@@ -179,14 +197,17 @@ class World:
             fe.SetField('flowSD', float(e[3]))
             line = ogr.Geometry(type=ogr.wkbLineString)
             try:
-                #print "creating edge lines"
+                # print "creating edge lines"
                 line.AddPoint(self.centdict[int(e[0])][0], self.centdict[int(e[0])][1])
                 line.AddPoint(self.centdict[int(e[1])][0], self.centdict[int(e[1])][1])
                 fe.SetGeometryDirectly(line)
                 el.CreateFeature(fe)
-            except: #node not in centdict
+            except:  # node not in centdict
                 pass
         el.SyncToDisk()
+        self.out_spatial_ref.MorphToESRI()
+        with open('Edges.prj', 'w') as fobj:
+            fobj.write(self.out_spatial_ref.ExportToWkt())
 
     def create_data_layer(self, varlist, data):
         """
@@ -209,56 +230,59 @@ class World:
             dsd = self.driver.CreateDataSource(os.path.join(self.outdir, 'Data.shp'))
             self.datasource = dsd
             dl = dsd.CreateLayer("sim_results", geom_type=ogr.wkbPolygon)
-            #Create the fields
+            # Create the fields
         fi1 = ogr.FieldDefn("geocode", field_type=ogr.OFTInteger)
         fin = ogr.FieldDefn("name", field_type=ogr.OFTString)
         fic = ogr.FieldDefn("colors", field_type=ogr.OFTString)
         dl.CreateField(fi1)
         dl.CreateField(fin)
-        dl.CreateField(fic) #json array with colors
+        dl.CreateField(fic)  # json array with colors
         for v in varlist:
-            #print "creating data fields"
+            # print "creating data fields"
             fi = ogr.FieldDefn(v, field_type=ogr.OFTReal)
             fi.SetPrecision(12)
             dl.CreateField(fi)
 
-        #Add the features (points)
+        # Add the features (points)
         for l in data:
-        #            print l
-            #Iterate over the lines of the data matrix.
+            #            print l
+            # Iterate over the lines of the data matrix.
             hexcolors = [self.get_hex_color(norms[i](v)) for i, v in enumerate(l[1:])]
-            gc = l[0]
+            gc = int(l[0])
             try:
                 geom = self.geomdict[gc]
-            except KeyError: #Geocode not in polygon dictionary
-                raise KeyError("Geocode %s not in polygon dictionary" % gc)
+            except KeyError:  # Geocode not in polygon dictionary
+                raise KeyError("Geocode %s not in polygon dictionary\n{}" % gc)
             if geom.GetGeometryType() != 3: continue
-            #print geom.GetGeometryCount()
+            # print geom.GetGeometryCount()
             fe = ogr.Feature(dl.GetLayerDefn())
             fe.SetField('geocode', gc)
             fe.SetField('name', self.namedict[gc])
             fe.SetField('colors', str(hexcolors))
             for v, d in zip(varlist, l[1:]):
-                #print v,d
+                # print v,d
                 fe.SetField(v, float(d))
-                #Add the geometry
-            #print "cloning geometry"
+                # Add the geometry
+                # print "cloning geometry"
             clone = geom.Clone()
-            #print geom
-            #print "setting geometry"
+            # print geom
+            # print "setting geometry"
             fe.SetGeometry(clone)
-            #print "creating geom"
+            # print "creating geom"
             dl.CreateFeature(fe)
 
         dl.SyncToDisk()
+        self.out_spatial_ref.MorphToESRI()
+        with open('Data.prj', 'w') as fobj:
+            fobj.write(self.out_spatial_ref.ExportToWkt())
         self.save_data_geojson(dl)
 
     def get_hex_color(self, value):
         cols = cm.get_cmap("YlOrRd", 256)
         rgba = cols(value * 256)
-        bgrcol = list(rgba[:-1]) #rgb(list)
-        bgrcol.reverse() #turn it into bgr
-        hexcol = "#80" + rgb2hex(bgrcol)[1:] #abgr Alpha set to 128
+        bgrcol = list(rgba[:-1])  # rgb(list)
+        bgrcol.reverse()  # turn it into bgr
+        hexcol = "#80" + rgb2hex(bgrcol)[1:]  # abgr Alpha set to 128
         return hexcol
 
     def save_data_geojson(self, dl, namefield=None):
@@ -272,7 +296,7 @@ class World:
 
         feature_collection = {"type": "FeatureCollection",
                               "features": []
-        }
+                              }
 
         #        if spatial_reference is not None:
         #            with open("data.crs", "wb") as f:
@@ -290,14 +314,14 @@ class World:
         fe = dl.GetNextFeature()
         #        print fe
         while fe is not None:
-        #            print namefield
-            fi = fe.GetField('name')
+            #            print namefield
+            fi = fe.GetField(namefield)
             #            print fi,type(fi)
             try:
                 fi = fi.decode('utf8', 'ignore')
             except AttributeError:
-                fi = '' #name is None
-            fe.SetField('name', str(fi))
+                fi = ''  # name is None
+            fe.SetField(namefield, str(fi))
             feature = json.loads(fe.ExportToJson())
             feature['properties']['colors'] = eval(feature['properties']['colors'])
             feature_collection["features"].append(feature)
@@ -306,7 +330,6 @@ class World:
         with open(os.path.join(self.outdir, 'data.json'), 'w') as f:
             json.dump(feature_collection, f)
 
-
     def gen_sites_file(self, fname):
         """
         This method generate a sites
@@ -320,23 +343,21 @@ class World:
                 y = self.centdict[gc][1]
                 name = fe.GetField(self.namefield)
                 line = "%s,%s,%s,%s\n" % (x, y, name, gc)
-                #fe.SetField('name',f.GetField(self.namefield))
+                # fe.SetField('name',f.GetField(self.namefield))
                 f.write(line)
 
-
     def close_sources(self):
         """
         Close the data sources so that data is flushed and and files are closed
         """
         if self.nodesource:
             self.nodesource.Destroy()
-            #print "closed node files"
+
         if self.edgesource:
             self.edgesource.Destroy()
-            #print "closed edge files"
+
         if self.datasource:
             self.datasource.Destroy()
-            #print "closed data files"
 
 
 class AnimatedKML(object):
@@ -377,9 +398,9 @@ class AnimatedKML(object):
         vals = array([i[2] for i in data])
         norm = Normalize(vals.min(), vals.max())
         for i, d in enumerate(data):
-        #            print i, " of ",  len(data)
+            #            print i, " of ",  len(data)
             pm = self.pmdict[d[0]]
-            #clone placemark to receive new data
+            # clone placemark to receive new data
             pm_newtime = pm.cloneNode(1)
             # Renaming placemark
             on = pm_newtime.getElementsByTagName('name')[0]
@@ -387,7 +408,7 @@ class AnimatedKML(object):
             nn.appendChild(self.kmlDoc.createTextNode(d[0] + '-' + str(d[1])))
             pm_newtime.replaceChild(nn, on)
             nl = pm_newtime.childNodes
-            #extrude polygon
+            # extrude polygon
             pol = pm_newtime.getElementsByTagName('Polygon')[0]
             alt = self.kmlDoc.createElement('altitudeMode')
             alt.appendChild(self.kmlDoc.createTextNode('relativeToGround'))
@@ -404,14 +425,14 @@ class AnimatedKML(object):
             #            ob.replaceChild(nlr, lr)
             ob.removeChild(lr)
             ob.appendChild(nlr)
-            #set polygon style
+            # set polygon style
             col = rgb2hex(cm.jet(norm(d[2]))[:3]) + 'ff'
-            st = pm_newtime.getElementsByTagName('Style')[0] #style
+            st = pm_newtime.getElementsByTagName('Style')[0]  # style
             nst = self.set_polygon_style(st, col)
             pm_newtime.removeChild(st)
             pm_newtime.appendChild(nst)
 
-            #add timestamp
+            # add timestamp
             ts = self.kmlDoc.createElement('TimeStamp')
             w = self.kmlDoc.createElement('when')
             w.appendChild(self.kmlDoc.createTextNode(str(d[1])))
@@ -444,13 +465,12 @@ class AnimatedKML(object):
         if self.extrude:
             lr.appendChild(ex)
             lr.appendChild(ts)
-        #        lr.appendChild(altoff)
+        # lr.appendChild(altoff)
         return lr
 
-
     def set_polygon_style(self, style, color):
         st = style
-        pst = st.getElementsByTagName('PolyStyle')[0] #polygon style
+        pst = st.getElementsByTagName('PolyStyle')[0]  # polygon style
         pst1 = self.kmlDoc.createElement('PolyStyle')
         pfill = self.kmlDoc.createElement('fill')
         pcol = self.kmlDoc.createElement('color')
@@ -471,7 +491,7 @@ class AnimatedKML(object):
             fname = self.fname.split('.')[0] + '_animation'
         else:
             fname = os.path.join(dir, fname)
-        #        ld = zlib.compress(self.kmlDoc.toxml('utf-8'))
+        # ld = zlib.compress(self.kmlDoc.toxml('utf-8'))
         with open(fname + '.kml', 'w') as f:
             f.write(self.kmlDoc.toprettyxml(indent='  ', encoding='utf-8'))
             # Now zip the kml to generate the kmz
@@ -488,7 +508,7 @@ class KmlGenerator:
 
     def __init__(self):
         self.doc = None
-        self.dnode = None #document node in the DOM
+        self.dnode = None  # document node in the DOM
         self.genRoot()
 
     def genRoot(self):
@@ -513,16 +533,16 @@ class KmlGenerator:
     def get_hex_color(self, value):
         jet = cm.get_cmap("jet", 50)
         rgba = jet(value * 50)
-        bgrcol = list(rgba[:-1]) #rgb(list)
-        bgrcol.reverse() #turn it into bgr
-        hexcol = "#80" + rgb2hex(bgrcol)[1:] #abgr Alpha set to 128
+        bgrcol = list(rgba[:-1])  # rgb(list)
+        bgrcol.reverse()  # turn it into bgr
+        hexcol = "#80" + rgb2hex(bgrcol)[1:]  # abgr Alpha set to 128
         return hexcol
 
     def addNodes(self, layer, names=None):
         """
         Adds a node to the document.
         d is the document element KML dom object.
-        layer is a layer with polygons
+        layer is a data layer with polygons and the results of a simulation as features
         names is a dictionary of names indexed by geocode(int)
         """
         doc = self.dnode
@@ -530,7 +550,7 @@ class KmlGenerator:
         layer.ResetReading()
         while 1:
             f = layer.GetNextFeature()
-            if not f:#exit after the last feature
+            if not f:  # exit after the last feature
                 break
             prevalence = float(f.GetField("prevalence"))
             hexcol = self.get_hex_color(prevalence)
@@ -547,12 +567,12 @@ class KmlGenerator:
                         name = ""
                 description = "Prevalence: %s;\nTotal cases: %s;\nImported Cases: %s;" % (
                     prevalence, f.GetField("totalcases"), f.GetField("arrivals"))
-                locale.setlocale(locale.LC_ALL, "C") #avoids conversion of decimal points
-                gml = g.ExportToGML() #extract the coordinates from the GML representation
+                locale.setlocale(locale.LC_ALL, "C")  # avoids conversion of decimal points
+                gml = g.ExportToGML()  # extract the coordinates from the GML representation
                 coords = gml.split('<gml:coordinates>')[1].split('</gml:coordinates>')[0]
-                coords = " ".join([i + ",0" for i in coords.split(" ")]) #add z coordinate
-                #create the kml elements
-                #placemark and sub elements
+                coords = " ".join([i + ",0" for i in coords.split(" ")])  # add z coordinate
+                # create the kml elements
+                # placemark and sub elements
                 pm = self.kmldoc.createElement("Placemark")
                 nm = self.kmldoc.createElement("name")
                 nm.appendChild(self.kmldoc.createTextNode(name))
@@ -560,7 +580,7 @@ class KmlGenerator:
                 desc.appendChild(self.kmldoc.createTextNode(description))
                 pm.appendChild(nm)
                 pm.appendChild(desc)
-                #style and subelements
+                # style and subelements
                 st = self.kmldoc.createElement("Style")
                 pm.appendChild(st)
                 ps = self.kmldoc.createElement("PolyStyle")
@@ -575,7 +595,7 @@ class KmlGenerator:
                 ps.appendChild(outline)
                 st.appendChild(ps)
                 doc.appendChild(pm)
-                #Multigeometry
+                # Multigeometry
                 mg = self.kmldoc.createElement("MultiGeometry")
                 pm.appendChild(mg)
                 polygon = self.kmldoc.createElement("Polygon")
@@ -598,6 +618,24 @@ class KmlGenerator:
         f.close()
 
 
+def gdal_error_handler(err_class, err_num, err_msg):
+    errtype = {
+        gdal.CE_None: 'None',
+        gdal.CE_Debug: 'Debug',
+        gdal.CE_Warning: 'Warning',
+        gdal.CE_Failure: 'Failure',
+        gdal.CE_Fatal: 'Fatal'
+    }
+    err_msg = err_msg.replace('\n', ' ')
+    err_class = errtype.get(err_class, 'None')
+    print 'Error Number: %s' % (err_num)
+    print 'Error Type: %s' % (err_class)
+    print 'Error Message: %s' % (err_msg)
+
+
+# install error handler
+gdal.PushErrorHandler(gdal_error_handler)
+
 if __name__ == "__main__":
     # opening data source
     w = World('riozonas_LatLong.shp', 'nome_zonas', 'zona_trafe')
@@ -606,7 +644,7 @@ if __name__ == "__main__":
     w.draw_layer(layer)
     w.save_data_geojson(layer)
     w.create_node_layer()
-    w.nodesource.Destroy() #flush data to disk
+    w.nodesource.Destroy()  # flush data to disk
 ##    k = KmlGenerator()
 ##    k.addNodes(w.datasource.GetLayer(0))
 ##    k.writeToFile()
