@@ -22,6 +22,8 @@ from data_io import *
 import epigdal
 import __version__
 import requests
+# import pydevd
+# pydevd.settrace('localhost', port=39347, stdoutToServer=True, stderrToServer=True)
 
 
 
@@ -137,13 +139,13 @@ class simulate:
             #SIMULATION AND OUTPUT
             self.steps = eval(config['simulation and output.steps'])
             self.outdir = config['simulation and output.outdir']
-            self.MySQLout = eval(config['simulation and output.mysqlout'])
+            self.SQLout = eval(config['simulation and output.sqlout'])
             #            self.Rep = eval(config['simulation and output.report'])
             #            self.siteRep = eval(config['simulation and output.siterep'])
             self.replicas = eval(config['simulation and output.replicas'])
             self.randomize_seeds = eval(config['simulation and output.randseed'])
             self.Batch = eval(config['simulation and output.batch'])
-        except KeyError, v:
+        except KeyError as v:
             V = v.__str__().split('.')
             sys.exit("Please check the syntax of your '.epg' file.\nVariable %s, from section %s was not specified." % (
                 V[1], V[0]))
@@ -319,12 +321,12 @@ class simulate:
             poplist = [log10(site.totpop) for site in self.g.site_dict.itervalues()]
             lpl = len(poplist)
             popprob = array(poplist) / sum(poplist)  #acceptance probability
-            u = floor(uniform(0, lpl, self.replicas))
+            u = floor(np.random.uniform(0, lpl, self.replicas))
             sites = []
             i = 0
             site_list = self.g.site_list
             while i < self.replicas:
-                p = uniform(0, 1)
+                p = np.random.uniform(0, 1)
                 if p <= popprob[int(u[i])]:
                     sites.append(site_list[int(u[i])])
                     i += 1
@@ -345,7 +347,7 @@ class simulate:
         self.seed = [(seed.geocode, seedvar, n)]
 
         for site in site_list:
-            if site.geocode == seed.geocode:
+            if site.geocode == seed.geocode or seed.geocode == 'all':
                 site.bi[seedvar] = n
                 #                site.ts[0][seedpos] = n
 
@@ -367,7 +369,7 @@ class simulate:
         self.runGraph(self.g, self.steps, transp=self.doTransp)
         elapsed = time.time() - start
         self.Say('Simulation lasted %s seconds.' % elapsed)
-        if self.MySQLout:
+        if self.SQLout:
             if self.backend == 'csv':
                 self.outToCsv(self.modelName)
             else:
@@ -385,7 +387,7 @@ class simulate:
         Saves the time series *site.ts* to outfile as specified on the
         model script.
         """
-        data = array(site.ts, Float)
+        data = array(site.ts, float)
         inc = site.incidence
         f = open(self.outdir + self.outfile, 'w')
         f.write('time,E,I,S,incidence\n')
@@ -913,7 +915,8 @@ class simulate:
         #saving Epistats
         self.Say('Saving Epidemiological results...')
         stats = [str(i) for i in self.g.getEpistats()]
-        seed = [s for s in self.g.site_dict.itervalues() if s.geocode == self.seed[0][0]][0]
+
+        seed = [s for s in self.g.site_dict.itervalues() if s.geocode == self.seed[0][0] or self.seed[0][0] == 'all'][0]
         stats.pop(1)  #Remove epispeed which is a vector
         if os.path.exists('epistats.csv'):
             stf = codecs.open('epistats.csv', 'a', self.encoding)  #append to file
@@ -1035,7 +1038,7 @@ class simulate:
             string + '\n'
 
 
-def storeSimulation(s, db='epigrass', host='localhost', port=3306):
+def storeSimulation(S, usr, passw, db='epigrass', host='localhost', port=3306):
     """
     store the Simulate object *s* in the epigrass database
     to allow distributed runs. Currently not working.
@@ -1047,7 +1050,7 @@ def storeSimulation(s, db='epigrass', host='localhost', port=3306):
     sql = """CREATE TABLE %s(
         `simulation` BLOB);""" % table
     Cursor.execute(sql)
-    blob = cPickle.dumps(s)
+    blob = cPickle.dumps(S)
     sql2 = 'INSERT INTO %s' % table + ' VALUES(%s)'
     Cursor.execute(sql2, blob)
     con.close()
@@ -1069,7 +1072,7 @@ def onStraightRun(args):
         repRuns(S)
 
     if S.Batch:
-        self.Say('Simulation Started.')
+        S.Say('Simulation Started.')
 
         # run the batch list
         for i in S.Batch:
@@ -1080,7 +1083,7 @@ def onStraightRun(args):
             # Generates the simulation object
             T = simulate(fname=i, host=S.host, user=S.usr, password=S.passw, backend=S.backend)
 
-            self.Say('starting model %s' % i)
+            S.Say('starting model %s' % i)
             T.start()  # Start the simulation
 
 
@@ -1097,12 +1100,12 @@ def repRuns(S):
     password = S.passw
     backend = S.backend
     nseeds = S.seed[0][2]  #number o individual to be used as seeds
-    self.Say("Replication type: ", randseed)
+    S.Say("Replication type: ", randseed)
     if randseed:
         seeds = S.randomizeSeed(randseed)
     reps = S.replicas
     for i in xrange(reps):
-        self.Say("Starting replicate number %s" % i)
+        S.Say("Starting replicate number %s" % i)
         S = simulate(fname=fname, host=host, user=user, password=password, backend=backend)
         if randseed:
             S.setSeed(seeds[i], nseeds)
@@ -1165,10 +1168,10 @@ def main():
                         help='Epigrass model definition file (.epg).')
 
     args = parser.parse_args()
-    if args.backend == "mysql" and not (options.dbuser and options.dbpass):
+    if args.backend == "mysql" and not (args.dbuser and args.dbpass):
         parser.error("You must specify a user and password when using MySQL.")
     if args.backend not in ['mysql', 'sqlite', 'csv']:
-        parser.error('"%s" is an invalid backend type.' % options.backend)
+        parser.error('"%s" is an invalid backend type.' % args.backend)
     print
     '==> ', args.epg
 
